﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.CompilerServices;

namespace NotifyPerfTests
{
	public class ViewModel : INotifyPropertyChanged
	{
		public event PropertyChangedEventHandler PropertyChanged;

		protected void NotifyPropertyChanged([CallerMemberName] string propertyName = "")
		{
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null)
			{
				handler(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		protected void NotifyPropertyChanged(PropertyChangedEventArgs args)
		{
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null)
			{
				handler(this, args);
			}
		}

		protected void NotifyPropertyChanged<T>(Expression<Func<T>> propertyExpression)
		{
			var propertyName = ExtractPropertyName(propertyExpression);
			this.NotifyPropertyChanged(propertyName);
		}

		public void NotifyObjectChanged()
		{
			this.NotifyPropertyChanged((string)null);
		}

		public static string ExtractPropertyName<T>(Expression<Func<T>> propertyExpression)
		{
			if (propertyExpression == null)
			{
				throw new ArgumentNullException("propertyExpression");
			}

			var memberExpression = propertyExpression.Body as MemberExpression;
			if (memberExpression == null)
			{
				throw new ArgumentException(@"The expression is not a member access expression", "propertyExpression");
			}

			var property = memberExpression.Member as PropertyInfo;
			if (property == null)
			{
				throw new ArgumentException(@"The member access expression does not access property", "propertyExpression");
			}

			var getMethod = property.GetMethod;
			if (getMethod.IsStatic)
			{
				throw new ArgumentException(@"The referenced property is a static property", "propertyExpression");
			}

			string propertyName = memberExpression.Member.Name;
			return propertyName;
		}
	}
}